==> antos0.1.tar.gz: antos0.1/signal.c <==
/*
 * signal.c
 *
 * Copyright 2004, Minoru Murashima. All rights reserved.
 * Distributed under the terms of the BSD License.
 */


#include<types.h>
#include<lib.h>
#include<proc.h>
#include<interrupt.h>
#include<mm.h>
#include<lock.h>
#include<errno.h>
#include<signal.h>


/****************************************************************************************
 *
 * < signal default actions >
 *
 *****************************************************************************************/

/*
 * PRIVATE
 * Default aactinos.
 */
/* terminate process. */
static void terminate(int signum)
{
	if(get_current_task()->nest_count==0)exit(0,signum);
}

/* abnormal terminate process. */
static void abnormalTerminate(int signum)
{
	if(get_current_task()->nest_count==0)exit(0,signum);
}

/* stop process. */
static void stopProcess(int signum)
{
	get_current_task()->signum=signum;
	del_from_schedule(TASK_SIGNAL_WAIT);
	wait_task();
}


/*
 * PUBLIC
 * signal actions.
 */
static SIG_ACTION defaultAction[]={
	NULL,
	abnormalTerminate,	/* SIGABRT		A */
	terminate,			/* SIGALRM		T */
	abnormalTerminate,	/* SIGBUS		A */
	SIG_IGN,			/* SIGCHLD		I */
	SIG_DFL,			/* SIGCONT		C */
	abnormalTerminate,	/* SIGFPE		A */
	terminate,			/* SIGHUP		T */
	abnormalTerminate,	/* SIGILL		A */
	terminate,			/* SIGINT		T */
	terminate,			/* SIGKILL		T */
	terminate,			/* SIGPIPE		T */
	terminate,			/* SIGPOLL		T */
	terminate,			/* SIGPROF		T */
	abnormalTerminate,	/* SIGQUIT		A */
	abnormalTerminate,	/* SIGSEGV		A */
	stopProcess,		/* SIGSTOP		S */
	abnormalTerminate,	/* SIGSYS		A */
	terminate,			/* SIGTERM		T */
	abnormalTerminate,	/* SIGTRAP		A */
	stopProcess,		/* SIGTSTP		S */
	stopProcess,		/* SIGTTIN		S */
	stopProcess,		/* SIGTTOU		S */
	SIG_IGN,			/* SIGURG		I */
	terminate,			/* SIGUSR1		T */
	terminate,			/* SIGUSR2		T */
	terminate,			/* SIGVTALRM	T */
	abnormalTerminate,	/* SIGXCPU		A */
	abnormalTerminate,	/* SIGXFSZ		A */
};


/****************************************************************************************
 *
 * < ʥ >
 *
 *****************************************************************************************/

/*
 * PRIVATE
 * ʥ빽¤Υɥ쥹顢ץɥ쥹롣
 * parameters : ʥ빽¤
 * return : ץ
 */
static inline PROC *sigToProc(SIGNAL *sig_st)
{
	return (PROC*)((uint)sig_st-(uint)&((PROC*)0)->signal_struct);
}


/*
 * PRIVATE
 * ʥ¹ԡ
 * parameters : user esp address
 */
static inline void doSignal(uint *user_esp)
{
	SIGNAL *sig_st=(SIGNAL*)get_current_task()->signal_struct;
	int i;


	if(sig_st->signal==0)return;

	/* ʥϥɥμ¹ԡ */
	for(i=0;sig_st->signal!=0;++i)
	{
		cli();

		if(sig_st->signal&(1<<i))
		{
			sig_st->signal^=(1<<i);
			sti();

			if((uint)sig_st->action[i]>=USER_BEG)
			{
				uint before_mask;

				/* ʥޥ촹 */
				before_mask=sig_st->mask;
				sig_st->mask|=sig_st->actMask[i]|(1<<i);

				/* 桼ϥɥ¹ԡ */
				doUserHandler(i,sig_st->action[i],*user_esp,&sig_st->beforeEsp,&sig_st->beforeStackEntry);
				sig_st->beforeEsp=0;

				/* ʥޥ᤹ */
				sig_st->mask=before_mask;
			}
			else if(sig_st->action[i]!=SIG_DFL)
				/* ǥեȥϥɥ¹ԡ */
				sig_st->action[i](i);
		}
		else sti();
	}
}


/*
 * PRIVATE
 * ̾ߤΥʥ¹ԡ
 * parameters : user esp
 */
void doSignalFromTrap(TRAP_FRAME frame)
{
	doSignal(&frame.user_esp);
}


/*
 * PRIVATE
 * 㳰ߤΥʥ¹ԡ
 * parameters : user esp
 */
void doSignalFromExcept(EXCEPT_FRAME frame)
{
	doSignal(&frame.user_esp);
}


/*
 * GLOBAL
 * ʥ
 * ǥեȥΡcontinue processפϡʥץ
 * ư롣
 * parameters : process,ʥ
 * return : 0 or error number
 */
int sendSignal(PROC *proc,int signal)
{
	int sig;
	SIGNAL *sig_st;


	sig_st=proc->signal_struct;

	/* ʥ뤬̵ˤƤʤ */
	if(sig_st->action[signal]==SIG_IGN)return -EPERM;

	sig_st->reserve|=(1<<signal)&sig_st->mask;		/* αΥʥ롣 */
	sig=((1<<signal)|sig_st->mask)^sig_st->mask;	/* αʤʥ롣 */

	if(sig!=0)
	{
		/* ʥԤʤ鵯 */
		add_to_schedule(proc,TASK_SIGNAL_WAIT);

		sig_st->signal|=sig;
	}

	return 0;
}


/*
 * GLOBAL
 * ʥ
 * parameters : process,ʥ
 * return : 0 or error=-1
 */
int forceSendSignal(PROC *proc,int signal)
{
	SIGNAL *sig_st;
	
	
	sig_st=proc->signal_struct;
	
	/* ʥԤʤ鵯 */
	add_to_schedule(sigToProc(sig_st),TASK_SIGNAL_WAIT);

	sig_st->signal|=1<<signal;

	return 0;
}


/*
 * GLOBAL
 * fork()Υʥ뵡ꡣ
 * parameters : ƥץΥʥ빽¤Ρ
 * return : 0 or error number
 */
SIGNAL *setSignal(SIGNAL *parent_sig)
{
	SIGNAL *sig_st;


	if((sig_st=kmalloc(sizeof(SIGNAL)))==NULL)return NULL;

	memcpy(sig_st->action,parent_sig->action,sizeof(SIGNAL));
	sig_st->signal=0;
	sig_st->reserve=0;
	sig_st->mask=parent_sig->mask;

	return sig_st;
}


/*
 * GLOBAL
 * exit()Υʥ뵡γ
 * parameters : ʥ빽¤
 */
void releaseSignal(SIGNAL *sig_st)
{
	kfree(sig_st);
}


/*
 * GLOBAL
 * ʥ뵡Υꥻåȡ
 * parameters : ʥ빽¤
 */
void resetSignal(SIGNAL *sig_st)
{
	/* ǥեȥ򥳥ԡ */
	memcpy(sig_st->action,defaultAction,SIG_ACTIONS*sizeof(SIG_ACTION));

	memset(sig_st->actMask,0,sizeof(uint)*SIG_ACTIONS);
	sig_st->signal=0;
	sig_st->reserve=0;
	sig_st->beforeEsp=0;
}


/*
 * GLOBAL
 * espͤ롣
 */
uint getBeforeEsp()
{
	return ((SIGNAL*)get_current_task()->signal_struct)->beforeEsp;
}


/*
 * GLOBAL
 * Υͥ륹åʪɥ쥹롣
 */
uint getBeforeStackEntry()
{
	return ((SIGNAL*)get_current_task()->signal_struct)->beforeStackEntry;
}


/*
 * GLOBAL
 * SIGINTȯ
 * return : ȯ=0ʳ
 */
int isSigint()
{
	return ((SIGNAL*)get_current_task()->signal_struct)->signal&1<<SIGINT;
} 


/*
 * GLOBAL
 * ʥ뵡ν
 * return : SIGNAL struct or NULL
 */
SIGNAL *initSignal()
{
	SIGNAL *sig_st;


	if((sig_st=kmalloc(sizeof(SIGNAL)))==NULL)return NULL;

	/*  */
	resetSignal(sig_st);
	sig_st->mask=0;

	return sig_st;
}


/****************************************************************************************
 *
 * < ƥॳ >
 *
 *****************************************************************************************/

int sys_sigaction(int signum,const struct sigaction *act,struct sigaction *oldact)
{
	SIGNAL *signal;
	struct sigaction tmp_act;


	if(signum==0)return 0;
	if((signum>=SIG_ACTIONS)||(signum<0))return -EINVAL;

	signal=(SIGNAL*)get_current_task()->signal_struct;
	tmp_act=*act;

	if(oldact!=NULL)
	{
		if(signal->action[signum]==SIG_IGN)oldact->sa_handler=SIG_IGN;
		else if(signal->action[signum]==defaultAction[signum])oldact->sa_handler=SIG_DFL;
		else oldact->sa_handler=signal->action[signum];

		oldact->sa_mask=signal->actMask[signum];
	}

	if(act!=NULL)
	{
		if((signum==SIGKILL)||(signum==SIGSTOP))return -EINVAL;		/* SIGKILLSIGSTOPѹԲġ */


		if(tmp_act.sa_handler==SIG_DFL)signal->action[signum]=defaultAction[signum];
		else if(tmp_act.sa_handler==SIG_IGN)signal->action[signum]=SIG_IGN;
		else if((uint)tmp_act.sa_handler>=USER_BEG)signal->action[signum]=tmp_act.sa_handler;
		else return -EINVAL;		/* ϥɥ饢ɥ쥹 */

		signal->actMask[signum]=tmp_act.sa_mask;
	}

	return 0;
}


int sys_sigprocmask(int how,uint *set,uint *oldset)
{
	uint tmp_mask=*set;
	SIGNAL *sig_st;


	sig_st=get_current_task()->signal_struct;

	if(oldset!=NULL)*oldset=sig_st->mask;

	switch(how)
	{
		case SIG_BLOCK:
			if(tmp_mask&((1<<SIGKILL)|(1<<SIGSTOP)))return -EINVAL;	/* SIGKILLSIGSTOPϥ֥åԲġ */
			sig_st->mask|=tmp_mask;
			break;
		case SIG_UNBLOCK:
			sig_st->mask&=~tmp_mask;
			break;
		case SIG_SETMASK:
			if(tmp_mask&((1<<SIGKILL)|(1<<SIGSTOP)))return -EINVAL;	/* SIGKILLSIGSTOPϥ֥åԲġ */
			sig_st->mask=tmp_mask;
			break;
	}

	return 0;
}


int sys_kill(int pid,int sig)
{
	int *lock_gate;
	int gid;
	int rest;
	PROC *proc;


	if((uint)sig>=SIG_ACTIONS)return -EPERM;

	if(pid>0)
	{
		lock_gate=getProcListRockGate(pid);
		enter_spinlock(lock_gate);
		{
			if((proc=searchProcList(pid))==NULL)
			{
				exit_spinlock(lock_gate);
				return -ESRCH;
			}
			rest=sendSignal(proc->signal_struct,sig);
		}
		exit_spinlock(lock_gate);
	}
	else if(pid==0)		/* callץƱ롼ס */
	{
		rest=-ESRCH;
		gid=get_current_task()->gid;

		lock_gate=getProcListRockGate(pid);
		enter_spinlock(lock_gate);
		{
			for(proc=NULL;(proc=getNextProc(gid,proc))!=NULL;)
				if((rest=sendSignal(proc->signal_struct,sig))<0)break;
		}
		exit_spinlock(lock_gate);
	}
	else if(pid==-1)	/* ץ */
	{
		rest=-ESRCH;

		lock_gate=getProcListRockGate(pid);
		enter_spinlock(lock_gate);
		{
			for(proc=NULL;(proc=getAllProc(proc))!=NULL;)
				if((rest=sendSignal(proc->signal_struct,sig))<0)break;
		}
		exit_spinlock(lock_gate);
	}
	else				/* pid<-1 -pidΥ롼ס */
	{
		rest=-ESRCH;
		gid=pid*-1;

		lock_gate=getProcListRockGate(pid);
		enter_spinlock(lock_gate);
		{
			for(proc=NULL;(proc=getNextProc(gid,proc))!=NULL;)
				if((rest=sendSignal(proc->signal_struct,sig))<0)break;
		}
		exit_spinlock(lock_gate);
	}

	return rest;
}


int sys_sigpending(sigset_t *set,int flag)
{
	SIGNAL *sig_st;


	sig_st=get_current_task()->signal_struct;

	switch(flag)
	{
		case SIG_PENDING_SOTORE:
			if(checkMem(set)==-1)return -EINVAL;
			*set=sig_st->reserve;
			break;
		case SIG_PENDING_SET:
			sig_st->reserve=*set;
			break;
	}

	return 0;
}


int sys_sigsuspend(const sigset_t *mask)
{
	uint before_mask;
	SIGNAL *sig_st;


	/* SIGKILLSIGSTOPϥ֥åԲġ */
	if(*mask&((1<<SIGKILL)|(1<<SIGSTOP)))return -EINVAL;

	sig_st=get_current_task()->signal_struct;
	before_mask=sig_st->mask;
	sig_st->mask=*mask;

	/* suspend. */
	del_from_schedule(TASK_SIGNAL_WAIT);
	wait_task();

	sig_st->mask=before_mask;

	return -EINTR;
}

==> antos0.2.tar.gz: antos0.2/signal.c <==
/*
 * signal.c
 *
 * Copyright 2004, Minoru Murashima. All rights reserved.
 * Distributed under the terms of the BSD License.
 */


#include<types.h>
#include<lib.h>
#include<proc.h>
#include<interrupt.h>
#include<mm.h>
#include<lock.h>
#include<errno.h>
#include<term.h>
#include<signal.h>


/****************************************************************************************
 *
 * < signal default actions >
 *
 *****************************************************************************************/

/*
 * PRIVATE
 * Default aactinos.
 */
/* terminate process. */
static void terminate(int signum)
{
	exit(0,signum);
}

/* abnormal terminate process. */
static void abnormalTerminate(int signum)
{
	exit(0,signum);
}

/* stop process. */
static void stopProcess(int signum)
{
	PROC *proc=get_current_task();


	proc->signum=signum;
/**************************************************************/
	/* ƥץwaitƤ鵯 */
	if((isForegrouond(proc))&&(proc->parent->state&TASK_CHILD_WAIT))
			add_to_schedule(proc->parent,TASK_SIGNAL_WAIT);
/***************************************************************/
	del_from_schedule(TASK_SIGNAL_WAIT);
	wait_task();
}


/*
 * PUBLIC
 * signal actions.
 */
static SIG_ACTION defaultAction[]={
	NULL,
	abnormalTerminate,	/* SIGABRT		A */
	terminate,			/* SIGALRM		T */
	abnormalTerminate,	/* SIGBUS		A */
	SIG_IGN,			/* SIGCHLD		I */
	SIG_DFL,			/* SIGCONT		C */
	abnormalTerminate,	/* SIGFPE		A */
	terminate,			/* SIGHUP		T */
	abnormalTerminate,	/* SIGILL		A */
	terminate,			/* SIGINT		T */
	terminate,			/* SIGKILL		T */
	terminate,			/* SIGPIPE		T */
	terminate,			/* SIGPOLL		T */
	terminate,			/* SIGPROF		T */
	abnormalTerminate,	/* SIGQUIT		A */
	abnormalTerminate,	/* SIGSEGV		A */
	stopProcess,		/* SIGSTOP		S */
	abnormalTerminate,	/* SIGSYS		A */
	terminate,			/* SIGTERM		T */
	abnormalTerminate,	/* SIGTRAP		A */
	stopProcess,		/* SIGTSTP		S */
	stopProcess,		/* SIGTTIN		S */
	stopProcess,		/* SIGTTOU		S */
	SIG_IGN,			/* SIGURG		I */
	terminate,			/* SIGUSR1		T */
	terminate,			/* SIGUSR2		T */
	terminate,			/* SIGVTALRM	T */
	abnormalTerminate,	/* SIGXCPU		A */
	abnormalTerminate,	/* SIGXFSZ		A */
};


/****************************************************************************************
 *
 * < ʥ >
 *
 *****************************************************************************************/

/*
 * PRIVATE
 * ʥ빽¤Υɥ쥹顢ץɥ쥹롣
 * parameters : ʥ빽¤
 * return : ץ
 */
static inline PROC *sigToProc(SIGNAL *sig_st)
{
	return (PROC*)((uint)sig_st-(uint)&((PROC*)0)->signal_struct);
}


/*
 * PRIVATE
 * ʥ¹ԡ
 * parameters : user esp address
 */
static inline void doSignal(uint *user_esp)
{
	SIGNAL *sig_st;
	PROC *proc=get_current_task();
	int i;


	if(proc->nest_count>0)return;
	
	sig_st=proc->signal_struct;

	/* ʥϥɥμ¹ԡ */
	while(sig_st->signal!=0)
	{
		for(i=0;(sig_st->signal&(1<<i))==0;++i);
		sig_st->signal^=(1<<i);		/* ʥ륯ꥢ*/

		if((uint)sig_st->action[i]>=USER_BEG)
		{
			uint before_mask;

			/* ʥޥ촹 */
			before_mask=sig_st->mask;
			sig_st->mask|=sig_st->actMask[i]|(1<<i);

			/* 桼ϥɥ¹ԡ */
			doUserHandler(i,sig_st->action[i],*user_esp,&sig_st->beforeEsp,&sig_st->beforeStackEntry);
			sig_st->beforeEsp=0;

			/* ʥޥ᤹ */
			sig_st->mask=before_mask;
		}
		else if(sig_st->action[i]!=SIG_DFL)
		{
			/* ǥեȥϥɥ¹ԡ */
			sig_st->action[i](i);
		}
	}
}


/*
 * PRIVATE
 * ̾ߤΥʥ¹ԡ
 * parameters : user esp
 */
void doSignalFromTrap(TRAP_FRAME frame)
{
	doSignal(&frame.user_esp);
}


/*
 * PRIVATE
 * 㳰ߤΥʥ¹ԡ
 * parameters : user esp
 */
void doSignalFromExcept(EXCEPT_FRAME frame)
{
	doSignal(&frame.user_esp);
}


/*
 * GLOBAL
 * ʥ
 * ǥեȥΡcontinue processפϡʥץ
 * ư롣
 * parameters : process,ʥ
 * return : 0 or error number
 */
int sendSignal(PROC *proc,int signal)
{
	int sig;
	SIGNAL *sig_st;


	sig_st=proc->signal_struct;

	/* ʥ뤬̵ˤƤʤ */
	if(sig_st->action[signal]==SIG_IGN)return -EPERM;

	sig_st->reserve|=(1<<signal)&sig_st->mask;		/* αΥʥ롣 */
	sig=((1<<signal)|sig_st->mask)^sig_st->mask;	/* αʤʥ롣 */

	if(sig!=0)
	{
		/* ʥԤʤ鵯 */
		add_to_schedule(proc,TASK_SIGNAL_WAIT);

		sig_st->signal|=sig;
	}

	return 0;
}


/*
 * GLOBAL
 * ʥ
 * parameters : process,ʥ
 * return : 0 or error=-1
 */
int forceSendSignal(PROC *proc,int signal)
{
	SIGNAL *sig_st;


	sig_st=proc->signal_struct;

	/* ʥԤʤ鵯 */
	add_to_schedule(sigToProc(sig_st),TASK_SIGNAL_WAIT);

	sig_st->signal|=1<<signal;

	return 0;
}


/*
 * GLOBAL
 * fork()Υʥ뵡ꡣ
 * parameters : ƥץΥʥ빽¤Ρ
 * return : 0 or error number
 */
SIGNAL *setSignal(SIGNAL *parent_sig)
{
	SIGNAL *sig_st;


	if((sig_st=kmalloc(sizeof(SIGNAL)))==NULL)return NULL;

	memcpy(sig_st->action,parent_sig->action,sizeof(SIGNAL));
	sig_st->signal=0;
	sig_st->reserve=0;
	sig_st->mask=parent_sig->mask;

	return sig_st;
}


/*
 * GLOBAL
 * exit()Υʥ뵡γ
 * parameters : ʥ빽¤
 */
void releaseSignal(SIGNAL *sig_st)
{
	kfree(sig_st);
}


/*
 * GLOBAL
 * ʥ뵡Υꥻåȡ
 * parameters : ʥ빽¤
 */
void resetSignal(SIGNAL *sig_st)
{
	/* ǥեȥ򥳥ԡ */
	memcpy(sig_st->action,defaultAction,SIG_ACTIONS*sizeof(SIG_ACTION));

	memset(sig_st->actMask,0,sizeof(uint)*SIG_ACTIONS);
	sig_st->signal=0;
	sig_st->reserve=0;
	sig_st->beforeEsp=0;
}


/*
 * GLOBAL
 * espͤ롣
 */
uint getBeforeEsp()
{
	return ((SIGNAL*)get_current_task()->signal_struct)->beforeEsp;
}


/*
 * GLOBAL
 * Υͥ륹åʪɥ쥹롣
 */
uint getBeforeStackEntry()
{
	return ((SIGNAL*)get_current_task()->signal_struct)->beforeStackEntry;
}


/*
 * GLOBAL
 * SIGINTȯ
 * return : ȯ=0ʳ
 */
int isSigint()
{
	return ((SIGNAL*)get_current_task()->signal_struct)->signal&1<<SIGINT;
}


/*
 * GLOBAL
 * ʥ뵡ν
 * return : SIGNAL struct or NULL
 */
SIGNAL *initSignal()
{
	SIGNAL *sig_st;


	if((sig_st=kmalloc(sizeof(SIGNAL)))==NULL)return NULL;

	/*  */
	resetSignal(sig_st);
	sig_st->mask=0;

	return sig_st;
}


/****************************************************************************************
 *
 * < ƥॳ >
 *
 *****************************************************************************************/

int sys_sigaction(int signum,const struct sigaction *act,struct sigaction *oldact)
{
	SIGNAL *signal;
	struct sigaction tmp_act;


	if(signum==0)return 0;
	if((signum>=SIG_ACTIONS)||(signum<0))return -EINVAL;

	signal=(SIGNAL*)get_current_task()->signal_struct;
	tmp_act=*act;

	if(oldact!=NULL)
	{
		if(signal->action[signum]==SIG_IGN)oldact->sa_handler=SIG_IGN;
		else if(signal->action[signum]==defaultAction[signum])oldact->sa_handler=SIG_DFL;
		else oldact->sa_handler=signal->action[signum];

		oldact->sa_mask=signal->actMask[signum];
	}

	if(act!=NULL)
	{
		if((signum==SIGKILL)||(signum==SIGSTOP))return -EINVAL;		/* SIGKILLSIGSTOPѹԲġ */


		if(tmp_act.sa_handler==SIG_DFL)signal->action[signum]=defaultAction[signum];
		else if(tmp_act.sa_handler==SIG_IGN)signal->action[signum]=SIG_IGN;
		else if((uint)tmp_act.sa_handler>=USER_BEG)signal->action[signum]=tmp_act.sa_handler;
		else return -EINVAL;		/* ϥɥ饢ɥ쥹 */

		signal->actMask[signum]=tmp_act.sa_mask;
	}

	return 0;
}


int sys_sigprocmask(int how,uint *set,uint *oldset)
{
	uint tmp_mask=*set;
	SIGNAL *sig_st;


	sig_st=get_current_task()->signal_struct;

	if(oldset!=NULL)*oldset=sig_st->mask;

	switch(how)
	{
		case SIG_BLOCK:
			if(tmp_mask&((1<<SIGKILL)|(1<<SIGSTOP)))return -EINVAL;	/* SIGKILLSIGSTOPϥ֥åԲġ */
			sig_st->mask|=tmp_mask;
			break;
		case SIG_UNBLOCK:
			sig_st->mask&=~tmp_mask;
			break;
		case SIG_SETMASK:
			if(tmp_mask&((1<<SIGKILL)|(1<<SIGSTOP)))return -EINVAL;	/* SIGKILLSIGSTOPϥ֥åԲġ */
			sig_st->mask=tmp_mask;
			break;
	}

	return 0;
}


int sys_kill(int pid,int sig)
{
	int gid;
	int rest;
	PROC *proc;


	if((uint)sig>=SIG_ACTIONS)return -EPERM;

	if(pid>0)
	{
		if((proc=searchProcList(pid))==NULL)return -ESRCH;
		rest=sendSignal(proc,sig);
	}
	else if(pid==0)		/* callץƱ롼ס */
	{
		rest=-ESRCH;
		proc=get_current_task();
		gid=proc->gid;
		while((proc=getNextProc(proc,gid))!=NULL)
			if((rest=sendSignal(proc,sig))<0)break;
	}
	else if(pid==-1)	/* ץ */
	{
		rest=-ESRCH;
		for(proc=NULL;(proc=getAllProc(proc))!=NULL;)
			if((rest=sendSignal(proc,sig))<0)break;
	}
	else				/* pid<-1 -pidΥ롼ס */
	{
		rest=-ESRCH;
		gid=-pid;
		for(proc=NULL;(proc=getNextProc(proc,gid))!=NULL;)
			if((rest=sendSignal(proc,sig))<0)break;
	}

	return rest;
}


int sys_sigpending(sigset_t *set,int flag)
{
	SIGNAL *sig_st;


	sig_st=get_current_task()->signal_struct;

	switch(flag)
	{
		case SIG_PENDING_SOTORE:
			if(checkMem(set)==-1)return -EINVAL;
			*set=sig_st->reserve;
			break;
		case SIG_PENDING_SET:
			sig_st->reserve=*set;
			break;
	}

	return 0;
}


int sys_sigsuspend(const sigset_t *mask)
{
	uint before_mask;
	SIGNAL *sig_st;


	/* SIGKILLSIGSTOPϥ֥åԲġ */
	if(*mask&((1<<SIGKILL)|(1<<SIGSTOP)))return -EINVAL;

	sig_st=get_current_task()->signal_struct;
	before_mask=sig_st->mask;
	sig_st->mask=*mask;

	/* suspend. */
	del_from_schedule(TASK_SIGNAL_WAIT);
	wait_task();

	sig_st->mask=before_mask;

	return -EINTR;
}
